@@ -1,16 +1,18 @@
 #' Retrieve the taxonomic hierarchy for a given taxon ID.
 #' 
-#' @import XML RCurl plyr
+#' @import XML RCurl plyr rgbif
 #' 
 #' @param x character; taxons to query.
 #' @param db character; database to query. either \code{ncbi}, \code{itis}, 
-#'    \code{eol}, \code{col} or \code{tropicos}.
+#'    \code{eol}, \code{col}, \code{tropicos}, \code{gbif}.
 #' @param id character; identifiers, returned by \code{\link[taxize]{get_tsn}}, 
 #'    \code{\link[taxize]{get_uid}}, \code{\link[taxize]{get_eolid}}, 
-#'    \code{\link[taxize]{get_colid}}, or \code{\link[taxize]{get_tpsid}}.
+#'    \code{\link[taxize]{get_colid}}, \code{\link[taxize]{get_tpsid}}, 
+#'    \code{\link[taxize]{get_gbifid}}.
 #' @param ... Other arguments passed to \code{\link[taxize]{get_tsn}}, 
 #'    \code{\link[taxize]{get_uid}}, \code{\link[taxize]{get_eolid}}, 
-#'    \code{\link[taxize]{get_colid}}, or \code{\link[taxize]{get_tpsid}}.
+#'    \code{\link[taxize]{get_colid}}, \code{\link[taxize]{get_tpsid}},
+#'    \code{\link[taxize]{get_gbifid}}.
 #' @param start The first record to return. If omitted, the results are returned 
 #' 		from the first record (start=0). This is useful if the total number of 
 #' 		results is larger than the maximum number of results returned by a single 
@@ -29,7 +31,7 @@
 #' 
 #' @seealso \code{\link[taxize]{get_tsn}}, \code{\link[taxize]{get_uid}}, 
 #'    \code{\link[taxize]{get_eolid}}, \code{\link[taxize]{get_colid}}, 
-#'    \code{\link[taxize]{get_tpsid}}
+#'    \code{\link[taxize]{get_tpsid}}, \code{\link[taxize]{get_gbifid}}
 #' 
 #' @export
 #' @examples \dontrun{
@@ -38,6 +40,7 @@
 #' classification(c("Chironomus riparius", "aaa vva"), db = 'itis')
 #' classification(c("Chironomus riparius", "aaa vva"), db = 'eol')
 #' classification(c("Chironomus riparius", "aaa vva"), db = 'col')
+#' classification(c("Chironomus riparius", "asdfasdfsfdfsd"), db = 'gbif')
 #' classification(c("Poa annua", "aaa vva"), db = 'tropicos')
 #' 
 #' # Use methods for get_uid, get_tsn, get_eolid, get_colid, get_tpsid
@@ -50,14 +53,36 @@
 #' classification(get_tpsid(c("Poa annua", "aaa vva")))
 #' 
 #' # Pass many ids from class "ids"
-#' out <- get_ids(names="Poa annua", db = c('ncbi','itis','col','eol','tropicos'))
+#' out <- get_ids(names="Puma concolor", db = c('ncbi','gbif'))
 #' cl <- classification(out)
 #' 
-#' # Bind length-wise
+#' # Bind width-wise from class classification_ids
 #' cbind(cl)
 #' 
-#' # Bind width-wise
+#' # Bind length-wise
 #' rbind(cl)
+#' 
+#' # Many names to get_ids
+#' out <- get_ids(names=c("Puma concolor","Accipiter striatus"), db = c('ncbi','itis','col'))
+#' cl <- classification(out)
+#' rbind(cl)
+#' cbind(cl)
+#' 
+#' # rbind and cbind on class classification (from a call to get_colid, get_tsn, etc. 
+#' # - other than get_ids)
+#' cl_col <- classification(get_colid(c("Puma concolor","Accipiter striatus")))
+#' cl_uid <- classification(get_uid(c("Puma concolor","Accipiter striatus")))
+#' cl_tsn <- classification(get_tsn(c("Puma concolor","Accipiter striatus")))
+#' rbind(cl_col)
+#' rbind(cl_uid)
+#' rbind(cl_tsn)
+#' cbind(cl_col)
+#' cbind(cl_uid)
+#' cbind(cl_tsn)
+#' 
+#' tsns <- get_tsn(c("Puma concolor","Accipiter striatus"))
+#' cl_tsns <- classification(tsns)
+#' cbind(cl_tsns)
 #' }
 #' 
 #' @examples \donttest{
@@ -99,6 +124,11 @@ classification.default <- function(x, db = NULL, ...){
     out <- classification(id, ...)
     names(out) <- x
   }
+  if (db == 'gbif') {
+    id <- get_gbifid(x, ...)
+    out <- classification(id, ...)
+    names(out) <- x
+  }
   return(out)
 }
 
@@ -120,6 +150,7 @@ classification.tsn <- function(id, ...)
     }
   }
   out <- lapply(id, fun)
+  names(out) <- id
   class(out) <- 'classification'
   attr(out, 'db') <- 'itis'
   return(out)
@@ -153,6 +184,7 @@ classification.uid <- function(id, ...) {
     return(out)
   }
   out <- lapply(id, fun)
+  names(out) <- id
   class(out) <- 'classification'
   attr(out, 'db') <- 'ncbi'
   return(out)
@@ -185,9 +217,9 @@ classification.eolid <- function(id, key = NULL, callopts = list(), ...) {
     }
   }
   out <- lapply(id, fun)
+  names(out) <- id
   class(out) <- 'classification'
   attr(out, 'db') <- 'eol'
-#   names(out) <- id
   return(out)
 }
 
@@ -253,11 +285,35 @@ classification.tpsid <- function(id, key = NULL, callopts = list(), ...) {
     return(out)
   }
   out <- lapply(id, fun)
-  class(out) <- 'classification'
   names(out) <- id
+  class(out) <- 'classification'
   return(out)
 }
 
+#' @method classification gbifid
+#' @export
+#' @rdname classification
+classification.gbifid <- function(id, callopts = list(), ...) {
+  fun <- function(x){
+    if(is.na(x)) {
+      out <- NA
+    } else {
+      out <- suppressWarnings(tryCatch(name_usage(key = x, ...), error=function(e) e))
+      if(is(out, "simpleError")){ 
+        df <- NA
+      } else {
+#         out <- do.call(rbind.fill, lapply(out, data.frame))[,c('ScientificName','Rank')]
+        df <- ldply(out[c('kingdom','phylum','clazz','order','family','genus','species')])
+        df <- data.frame(name=df$V1, rank=df$.id)
+      }
+    }
+    return( df )
+  }
+  out <- lapply(id, fun)
+  names(out) <- id
+  class(out) <- 'classification'
+  return(out)
+}
 
 #' @method classification ids
 #' @export
@@ -281,16 +337,16 @@ classification.ids <- function(id, ...)
 #' @method cbind classification
 #' @export
 #' @rdname classification
-cbind.classification <- function(...)
+cbind.classification <- function(x)
 {
   gethiernames <- function(x){
     x$name <- as.character(x$name)
     x$rank <- as.character(x$rank)
     values <- data.frame(t(x[,'name']))
-    names(values) <- x[,'rank']
+    names(values) <- tolower(x[,'rank'])
     return( values )
   }
-  input <- c(...)
+  input <- x
   input <- input[sapply(input, class) %in% "data.frame"]
   do.call(rbind.fill, lapply(input, gethiernames))
 }
@@ -298,12 +354,13 @@ cbind.classification <- function(...)
 #' @method rbind classification
 #' @export
 #' @rdname classification
-rbind.classification <- function(...)
+rbind.classification <- function(x)
 {
-  input <- c(...)
-  input <- input[sapply(input, class) %in% "data.frame"]
-  df <- do.call(rbind, input)
-  df <- data.frame(names = gsub("[0-9]|\\.", "", row.names(df)), df)
+  input <- x
+  db <- attr(input, "db")
+  x <- input[vapply(x, class, "") %in% "data.frame"]
+  df <- do.call(rbind, x)
+  df <- data.frame(source = db, taxonid = gsub("\\.[0-9]+", "", row.names(df)), df)
   row.names(df) <- NULL
   return( df )
 }
@@ -315,20 +372,24 @@ cbind.classification_ids <- function(...)
 {
   input <- c(...)
   # remove non-data.frames
-  input <- input[sapply(input, function(x) class(x[[1]])) %in% "data.frame"]
+  input <- input[vapply(input, function(x) class(x[[1]]), "") %in% "data.frame"]
   
   gethiernames <- function(x){
     x$name <- as.character(x$name)
     x$rank <- as.character(x$rank)
     values <- data.frame(t(x[,'name']))
-    names(values) <- x[,'rank']
+    names(values) <- tolower(x[,'rank'])
     return( values )
   }
   do.call(rbind.fill, lapply(input, function(x){ 
-    tmp <- lapply(x, gethiernames)
-    do.call(rbind.fill, tmp)
-  })
+      tmp <- lapply(x, gethiernames)
+      do.call(rbind.fill, tmp)
+    })
   )
+#   # sort columns by rank order
+#   rank_ref$ranks[names(df) %in% tolower(rank_ref$ranks)]
+#   grep(names(values)[[2]], tolower(rank_ref$ranks))
+#   torank <- sapply(rank_ref[grep(downto, rank_ref$ranks):nrow(rank_ref),"ranks"], function(x) strsplit(x, ",")[[1]][[1]], USE.NAMES=F)
 }
 
 #' @method rbind classification_ids
@@ -338,10 +399,41 @@ rbind.classification_ids <- function(...)
 {
   input <- c(...)
   # remove non-data.frames
-  input <- input[sapply(input, function(x) class(x[[1]])) %in% "data.frame"]  
-  df <- do.call(rbind, lapply(input, function(x){
-    data.frame(names(x), x[[1]])
-  }))
-  row.names(df) <- NULL
-  return( df )
+  input <- input[vapply(input, function(x) class(x[[1]]), "") %in% "data.frame"]
+  #   df <- do.call(rbind, lapply(input, function(x){
+  # #             lapply(x, function(y) data.frame(names(y), y[[1]]))
+  #           coll <- list()
+  #           for(i in seq_along(x)){
+  #             coll[[i]] <- data.frame(names(x[i]), x[i][[1]])
+  #           }
+  #     do.call(rbind, coll)
+  #   }))
+  
+  df <- lapply(input, function(x){
+    coll <- list()
+    for(i in seq_along(x)){
+      coll[[i]] <- data.frame(names(x[i]), x[i][[1]])
+    }
+    coll
+  })
+  
+  get <- list()
+  for(i in seq_along(df[[1]])){
+    tmp <- do.call(rbind, lapply(df, "[[", i))
+    source2 <- gsub("\\.[0-9]+", "", row.names(tmp))
+    row.names(tmp) <- NULL
+    names(tmp)[1] <- "taxonid"
+    tmp <- data.frame(source = source2, tmp)
+    get[[i]] <- tmp
+  }
+  
+  if(length(get) == 1) 
+    get[[1]]
+  else
+    get
+  #   source2 <- gsub("\\.[0-9]+", "", row.names(df))
+  #   row.names(df) <- NULL
+  #   names(df)[1] <- "taxonid"
+  #   df <- data.frame(source = source2, df)
+#   return( res )
 }
\ No newline at end of file
