@@ -1,6 +1,6 @@
 #' Query Phylomatic for a phylogenetic tree.
 #' 
-#' @import httr ape
+#' @import httr ape stringr
 #' @param taxa Phylomatic format input of taxa names.
 #' @param taxnames If true, we get the family names for you to attach to your 
 #'    species names to send to Phylomatic API. If FALSE, you have to provide the 
@@ -15,30 +15,26 @@
 #' @param taxaformat Only option is slashpath for now. Leave as is.
 #' @param outformat One of newick, nexml, or fyt.
 #' @param clean Return a clean tree or not.
-#' @param parallel Run in parallel or not.
+#' @param db One of "ncbi", "itis", or "apg"
+#' @param verbose Print messages (default: TRUE).
 #' @details Use the web interface here http://phylodiversity.net/phylomatic/
 #' @return Newick formatted tree or nexml text.
 #' @examples \dontrun{ 
 #' # Input taxonomic names
 #' taxa <- c("Poa annua", "Phlox diffusa", "Helianthus annuus")
 #' tree <- phylomatic_tree(taxa=taxa, get = 'POST')
-#' plot(tree)
+#' plot(tree, no.margin=TRUE)
 #' 
 #' # Genus names
 #' taxa <- c("Poa", "Phlox", "Helianthus")
 #' tree <- phylomatic_tree(taxa=taxa, storedtree='R20120829', get='POST')
-#' plot(tree)
+#' plot(tree, no.margin=TRUE)
 #' 
 #' # Lots of names
 #' taxa <- c("Poa annua", "Collomia grandiflora", "Lilium lankongense", "Phlox diffusa", 
 #' "Iteadaphne caudata", "Gagea sarmentosa", "Helianthus annuus")
 #' tree <- phylomatic_tree(taxa=taxa, get = 'POST')
 #' plot(tree, no.margin=TRUE)
-#' 
-#' # In parallel with parallel=TRUE speeds up dramatically by doing the 
-#' # gathering of family names from NCBI in parallel
-#' registerDoMC(cores=4)
-#' phylomatic_tree(taxa=taxa, get='POST', parallel=TRUE)
 #'    
 #' # Output NeXML format
 #' taxa <- c("Gonocarpus leptothecus", "Gonocarpus leptothecus", "Lilium lankongense")
@@ -49,24 +45,19 @@
 #' @export
 
 phylomatic_tree <- function(taxa, taxnames = TRUE, get = 'GET',
-  informat = "newick", method = "phylomatic", storedtree = "smith2011", 
-  taxaformat = "slashpath", outformat = "newick", clean = "true", 
-  parallel=FALSE)
+  informat = "newick", method = "phylomatic", storedtree = "R20120829", 
+  taxaformat = "slashpath", outformat = "newick", clean = "true", db="apg", verbose=TRUE)
 {
   url = "http://phylodiversity.net/phylomatic/pmws"
   
   if(taxnames){
-    if(parallel){ 
-      dat_ <- llply(taxa, itis_phymat_format, format='isubmit', .parallel=TRUE)
-    } else {
-      dat_ <- llply(taxa, itis_phymat_format, format='isubmit')
-    }
+    dat_ <- phylomatic_format(taxa, format='isubmit', db=db)
     
     checknas <- sapply(dat_, function(x) strsplit(x, "/")[[1]][1])
-    checknas2 <- checknas[grep("na", checknas)]
-    if(length(checknas2)>0)
-      stop(sprintf("A family was not found for the following taxa:\n %s \n\n try setting taxnmaes=FALSE, and passing in a vector of strings, like \n%s", 
-                   paste(sapply(dat_, function(x) strsplit(x, "/")[[1]][3])[grep("na", checknas)], collapse=", "),
+    checknas2 <- checknas[match("na", checknas)]
+    if(is.numeric(checknas2))
+      stop(sprintf("A family was not found for the following taxa:\n %s \n\n try setting taxnames=FALSE, and passing in a vector of strings, like \n%s", 
+                   paste(sapply(dat_, function(x) strsplit(x, "/")[[1]][3])[match("na", checknas)], collapse=", "),
                    'phylomatic_tree(taxa = c("asteraceae/taraxacum/taraxacum_officinale", "ericaceae/gaylussacia/gaylussacia_baccata", "ericaceae/vaccinium/vaccinium_pallidum"), taxnames=FALSE, parallel=FALSE)'
       ))
     
@@ -79,23 +70,47 @@ phylomatic_tree <- function(taxa, taxnames = TRUE, get = 'GET',
                        outformat = outformat, clean = clean))
   
   if (get == 'POST') {  
-    tt <- POST(url, query=args)
+#     tt <- POST(url, body=list(taxa=dat_), query=args, multipart=FALSE)
+    out <- postForm(url, .params=args, style = "POST")
   } else if (get == 'GET') {
     tt <- GET(url, query=args)
+    stop_for_status(tt)
+    out <- content(tt, as="text")
   } else
   { stop("Error: get must be one of 'POST' or 'GET'") }
-  stop_for_status(tt)
-  out <- content(tt, as="text")
-  
-  outformat <- match.arg(outformat, choices=c("nexml",'newick'))
-  getnewick <- function(x){
-    tree <- gsub("\n", "", x[[1]])
-    read.tree(text = colldouble(tree))
+
+  if(grepl("No taxa in common", out)){
+    stop(out)
+  } else
+  {
+    # parse out missing taxa note
+    if(grepl("\\[NOTE: ", out)){
+      taxa_na <- str_extract(out, "NOTE:.+")
+      taxa_na2 <- str_extract(taxa_na, ":\\s[A-Za-z].+")
+      taxa_na2 <- strsplit(taxa_na2, ",")[[1]][-length(strsplit(taxa_na2, ",")[[1]])]
+      taxa_na2 <- gsub(":|\\s", "", taxa_na2)
+      taxa_na2 <- sapply(taxa_na2, function(x) strsplit(x, "/")[[1]][[3]], USE.NAMES=FALSE)
+      taxa_na2 <- taxize_capwords(gsub("_", " ", taxa_na2), onlyfirst=TRUE)
+      
+      mssg(verbose, taxa_na)
+      out <- gsub("\\[NOTE:.+", ";\n", out)
+    } else {
+        taxa_na2 <- NULL
+    }
+    
+    outformat <- match.arg(outformat, choices=c("nexml",'newick'))
+    getnewick <- function(x){
+      tree <- gsub("\n", "", x[[1]])
+      read.tree(text = colldouble(tree))
+    }
+    
+    res <- switch(outformat, 
+           nexml = out,
+           newick = getnewick(out))
+    class(res) <- c("phylo","phylomatic")
+    attr(res, "missing") <- taxa_na2
+    return( res )
   }
-  
-  switch(outformat, 
-         nexml = out,
-         newick = getnewick(out))
 }
 
 collapse_double_root <- function(y) {
@@ -120,4 +135,4 @@ colldouble <- function(z) {
   { treephylo <- collapse_double_root(z) } else
   { treephylo <- z }
   return(treephylo)
-}
\ No newline at end of file
+}
